package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.*;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.mes.bean.MesProdMouldRecord;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.IMesMouldMappingCfgRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProdMouldRecordRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 生成开模记录工步
 * @Author : zxw
 **/
@Slf4j
@Service("mesMouldRecordGenerateStepService")
public class MesMouldRecordGenerateStepService extends BaseStepService {

    @Autowired
    private MesProdMouldRecordRepository mesProdMouldRecordRepository;

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesMouldMappingCfgRepository mesMouldMappingCfgRepository;
    @Autowired
    private IMesProductionDispatchContextStepService mesProductionDispatchContextStepService;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getEquipmentVariableList(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue());

        //List<MesProductionPartContext> productionPartContextList = mesProductionDispatchContextStepService.getProductionPartContext(reqBean);
        // 获取条码信息
        List<MesProductionPsOutContext> mesProduceSnContexts = mesProductionDispatchContextStepService.getProductionPsOutContext(reqBean);
        // 获取头道模具号
        MesEquipVariableCollectContext mesFirstMouldNoCollectContext = mesProductionDispatchContextStepService.getFirstMouldNoContext(reqBean);
        // 获取模具号
        MesEquipVariableCollectContext mesMouldNoCollectContext = mesProductionDispatchContextStepService.getMouldNoContext(reqBean);
        /**
         * 保存开模记录
         */
        saveMouldRecord(reqBean, resultBean, productionProcessContext, mesProduceSnContexts, mesFirstMouldNoCollectContext, mesMouldNoCollectContext);

        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(MesExtEnumUtil.WORK_CELL_SCAN_MONITOR_LOG_TYPE.PROCESS.getValue()), stepResult, "生成开模记录成功!");
    }

    private void saveMouldRecord(StationRequestBean reqBean, StationResultBean resultBean, MesProductionProcessContext productionProcessContext, List<MesProductionPsOutContext> mesProduceSns, MesEquipVariableCollectContext mesFirstMouldNoCollectContext, MesEquipVariableCollectContext mesMouldNoCollectContext) {
        List<MesProdMouldRecord> mesProdMouldRecords = new ArrayList<>();

        MesScanMonitorContext mesScanMonitorContext = productionProcessContextStepService.doHandleScanMonitorContext(reqBean, true);

        Long moduleId = mesScanMonitorContext.getMouldRecordId();
        // 获取工位设备信息
        MesCellEquipContext mesCellEquipContext = productionProcessContext.getCurCellEquip();

        String equipmentCode = mesCellEquipContext.getEquipmentCode();

        String mouldNo = "";

        if (mesFirstMouldNoCollectContext != null) {
            mouldNo = mesFirstMouldNoCollectContext.getEquipVariableValue();
        } else if (mesFirstMouldNoCollectContext == null && mesMouldNoCollectContext != null) {
            mouldNo = mesMouldNoCollectContext.getEquipVariableValue();
        }

        MesProdMouldRecord mesProductionRecord = getMesProdMouldRecord(mesCellEquipContext, mouldNo, moduleId);

        ConvertBean.serviceModelInitialize(mesProductionRecord, reqBean.getUserInfo());
        mesProdMouldRecords.add(mesProductionRecord);


        mesProdMouldRecordRepository.saveAll(mesProdMouldRecords);
    }

    private MesProdMouldRecord getMesProdMouldRecord(MesCellEquipContext mesCellEquipContext, String mouldNo, Long mouldId) {
        MesProdMouldRecord mesProdMouldRecord = new MesProdMouldRecord();
        mesProdMouldRecord.setEquipId(mesCellEquipContext.getEquipId());
        mesProdMouldRecord.setId(mouldId);
      /*  mesProdMouldRecord.setMeterName(mesMouldMappingCfg == null ? null : mesMouldMappingCfg.getMeterName());
        mesProdMouldRecord.setAssetNum(mesMouldMappingCfg == null ? null : mesMouldMappingCfg.getAssetNum());*/
        mesProdMouldRecord.setMouldNo(mouldNo);
        mesProdMouldRecord.setEquipmentCode(mesCellEquipContext.getEquipmentCode());
        return mesProdMouldRecord;
    }
}
