package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.context;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.actor.shipping.dispatch.IFsmCommonService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.process.BaseProcessMonitorService;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

@Service
@Primary
public class MesProductionProcessMonitorService extends BaseProcessMonitorService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private IFsmCommonService fsmCommonService;

    @Override
    public Boolean doProcessStart(StationRequestBean requestBean) {
        //保存上下文工位扫描监控信息
        return productionProcessContextStepService.saveScanMonitorContext(requestBean);
    }

    @Override
    public Boolean doProcessComplete(StationRequestBean requestBean) {
        //执行跳过工序二开实现逻辑
        doJumpProcess(requestBean);
        //发送工序完成音
        this.sendMessage(requestBean, new StationResultBean().resultObj(MesPcnExtConstWords.PROCESS_COMPLETE).busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.CUSTOM_COMPONENT.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.SOUND.getValue()));
        return true;
    }

    @Override
    public Boolean doJumpProcess(StationRequestBean requestBean) {
        //清除上下文中的所有业务数据
        productionDispatchContextStepService.doFlushProductionDispatchContext(requestBean);
        //清除工步客制化区域数据【装配件清单】
        this.sendMessage(requestBean, new StationResultBean().busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.STEP_CUSTOM_CONTENT.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.TABLES.getValue()));
        return true;
    }

    @Override
    public Boolean doOffLine(StationRequestBean requestBean) {
        //删除当前状态点
        fsmCommonService.doFlushCache(requestBean);
        //清除上下文中的所有业务数据
        productionDispatchContextStepService.doFlushProductionDispatchContext(requestBean);
        //清除上下文中的所有基础数据
        productionProcessContextStepService.doFlushProductionProcessContext(requestBean);
        return true;
    }

    @Override
    public Boolean fsmReset(StationRequestBean requestBean) {
        //清除上下文中的所有业务数据
        productionDispatchContextStepService.doFlushProductionDispatchContext(requestBean);
        //清除上下文中的所有基础数据
        productionProcessContextStepService.doFlushProductionProcessContext(requestBean);
        //清除工步客制化区域数据【装配件清单】
        this.sendMessage(requestBean, new StationResultBean().busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.STEP_CUSTOM_CONTENT.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.TABLES.getValue()));
        return true;
    }

}
